local file = {priority = 0}

Player = {}

function Player:new( )
	local inst = Class:init( Player, Pawn, { 3,0,3 } )
	
	inst.cam = {
		eyeLevel = .46,
		height = .56,
		yaw = 0,
		pitch = 0,
		roll = 0
	}
	
	inst.weapon = weapon_pistol
	
	inst.health = 100
	inst.maxHealth = 100
	inst.armor = 0
	inst.maxArmor = 100
	inst.armorMult = .33
	
	inst.isPlayer = true
	inst.friction = 6
	inst.stiction = 6 * 100/320
	inst.acceleration = 10
	inst.maxSpeed = 3.2
	inst.velocity = { x = 0, y = 0, z = 0 }
	inst.gravity = 8
	inst.jumpvel = 2.7
	
	inst.stepTimer = 0
	inst.stepTimerMax = .25
	inst.stepSound = Sound:new( "assets/audio/footstep.wav", .14 )
	inst.stepSound:setRelative( true )

	inst.hitSound = Sound:new( "assets/audio/playerdamage.wav", 1 )
	inst.hitSound:setRelative( true )
	
	inst.attackTimer = 0
	inst.chargeTimer = 0
	inst.switchWeaponTimer = 0
	inst.switchWeaponTimerMax = .5
	inst.nextWeapon = inst.weapon
	inst.weaponIndex = 1
	
	inst.meleeTimer = 0
	inst.meleeTimerMax = 1
	inst.meleeTime = .4
	inst.meleeDir = 0
	inst.meleeSpeed = 0
	inst.meleeSpeedMax = 3.5
	inst.meleeDeacceleration = 8
	inst.meleeRange = .5
	inst.meleeDamage = 10
	inst.meleeJump = -.7
	inst.meleeGolden = false
	inst.meleeGoldenDamage = 80
	inst.meleeGoldenTime = .55
	inst.meleeSound = Sound:new( "assets/audio/melee.wav", 1 )
	inst.meleeSound:setRelative( true )
	
	inst.ammo = {
	{ name = "bullets", count = 300, maxCount = 300 },
	{ name = "shells", count = 50, maxCount = 50 },
	{ name = "energy", count = 300, maxCount = 300 },
	{ name = "explosives", count = 30, maxCount = 30 }
	}
	
	inst.weapons = {
	"Pistol",
	}
	
	inst.keys = {}
	
	return inst
end


function Player.hasKey( self, key )
	for i, k in pairs( self.keys ) do
		if k == key then
			return k
		end
	end
	
	return false
end

function Player.pickupKey( self, key )
	table.insert( self.keys, key )
end

function Player.setGoldenMelee( self, golden )
	self.meleeGolden = golden
	
	if golden then
		ViewModel.meleeModel = mod_goldenMelee
	else
		ViewModel.meleeModel = mod_melee
	end
end

function Player.setToStartState( self )
	self.health = self.maxHealth
	self.armor = 0
	self:setGoldenMelee( false )

	self.weapons = { "Pistol" }
	self.keys = {}

	self.ammo[1].count = 100
	self.ammo[2].count = 0
	self.ammo[3].count = 0
	self.ammo[4].count = 0

	self:switchWeapon( 1 )
end

function Player.look( self )
	-- capture the mouse
    lm.setRelativeMode(true)

	local yaw,pitch = Controller:getCameraYawPitch()
    self.cam.yaw = yaw
    self.cam.pitch = pitch

	-- roll
	local roll = self.cam.roll + Juice:getRotation() + .00001
	roll = roll * sin( self.cam.pitch + rad( 90 ) )

	g3d.camera.up = { cos( self.cam.yaw ) * roll, -sin( self.cam.pitch + rad( 90 ) ) / abs( roll ), -sin( self.cam.yaw ) * roll }
end

function Player.walk( self )
	local air = (self.y<0) or Controller:isJumping()
	local spd = self.maxSpeed
	local frc = air and 0 or self.friction 
	local stc = air and 0 or self.stiction*spd
	local acc = air and spd*self.acceleration/10 or spd*self.acceleration
	local yaw = self.cam.yaw
	local dir = Controller:getMoveDir()
	local vx0, vz0 = self.velocity.x , self.velocity.z
	local vzx = math.dist(vx0,vz0,0,0)

	local f0, f1, f2 = 1, hdt, hdt*hdt/2 -- parabolic
	if frc > 0 then -- stokes
		f0 = math.exp(  - frc * hdt )
		f1 = ( 1  - f0) / frc
		f2 = (hdt - f1) / frc
	end	
	local vx , vz = f0*vx0 , f0*vz0 
	local dx , dz = f1*vx0 , f1*vz0
	if dir and self.meleeTimer == 1 then -- restrict accel based on projected (real) power
		local ang = dir + yaw
		local axn, azn = cos(ang) , -sin(ang)
		local accScale = clamp( 0 , (spd-(vx0*axn + vz0*azn)*f0)/f1 , acc)
		local ax , az = accScale * axn , accScale * azn
		vx , vz = vx + f1*ax , vz + f1*az
		dx , dz = dx + f2*ax , dz + f2*az
	elseif frc*frc*vzx < stc*stc then -- override stokes with stiction
		local ang = math.angle(0,0,vx0,vz0)
		local vel = max(0, vzx - stc*hdt)
		local del = (vel+vzx)*hdt/2
		vx , vz = vel*cos(ang) , vel*sin(ang)
		dx , dz = del*cos(ang) , del*sin(ang)
	end
	self.velocity.x = vx
	self.velocity.z = vz
	self:move(dx,0,dz)
	self.collisionBox:setPos( self.x,self.y,self.z )
	
	
	self.stepTimer = self.stepTimer + math.dist(vx,vz,0,0) / spd * hdt
	if self.stepTimer > self.stepTimerMax then
		self.stepTimer = self.stepTimer - self.stepTimerMax
		if not air then
			self.stepSound:setPitch( 1 + random() * rsign() * .1 )
			self.stepSound:play()
		end
	end
end

function Player.viewBob( self )
	local speed = math.dist( 0,0, Player.velocity.x,Player.velocity.z ) / Player.maxSpeed
	local bobFrequency = 1 / Player.stepTimerMax * 2
	local bobFactor = .025
	local rollFactor = .07
	local bob = sin( time * bobFrequency ) * speed

	Player.cam.height = Player.cam.eyeLevel + abs( bob ) * bobFactor * speed
	Player.cam.roll = bob * rollFactor
end

function Player.fall( self )
	local vy0 = self.velocity.y
	self.velocity.y = vy0 + self.gravity * hdt -- positive y is downwards

	self:move(
		0,
		vy0*hdt + self.gravity*hdt*hdt/2,
		0
	)
	
	if self.y > 0 then -- sunken below ground
		self:setPos( self.x,0,self.z )
		self.collisionBox:setPos( self.x, 0, self.z )
		self.velocity.y = 0
	end

	if Controller:isJumping() and self.y>=0 then self.velocity.y = -self.jumpvel end
		

end

function Player.parentCamera( self )
	local x,y,z = self.x,self.y - self.cam.height,self.z
	local jx,jy,jz = Juice:getOffset()
	x = x + jx
	y = y + jy
	z = z + jz

	g3d.camera.lookInDirection( x,y,z, self.cam.yaw,self.cam.pitch )
	g3d.camera.focal = Controller:getFocal() / Juice:getScale()
	
	la.setPosition( x,y,z )
	
	local vx,vy,vz = g3d.camera.getLookVector()
	g3d.camera.lookInDirection( nil,nil,nil, self.cam.yaw,self.cam.pitch + rad(90) )
	local uvx,uvy,uvz = g3d.camera.getLookVector()
	g3d.camera.lookInDirection( nil,nil,nil, self.cam.yaw,self.cam.pitch )
	
	la.setOrientation( vx,vy,vz, uvx,uvy,uvz )
	
	g3d.camera.updateProjectionMatrix()
end

function Player.fire( self )
	local particleOffset = .15
	
	self.attackTimer = min( self.weapon.attackTimerMax, self.attackTimer + hdt )
	
	if Controller:isFiring() and	-- charge
	self.attackTimer == self.weapon.attackTimerMax and
	self.switchWeaponTimer == 0 and
	self.meleeTimer == self.meleeTimerMax and
	Player.ammo[Player.weapon.ammoIndex].count >= Player.weapon.ammoCost then
		Player.chargeTimer = min( Player.weapon.chargeTime, Player.chargeTimer + hdt )
	else
		Player.chargeTimer = max( 0, Player.chargeTimer - hdt )
	end

	if Player.weapon.chargeSound then
		if Player.chargeTimer > 0 then
			Player.weapon.chargeSound.source:setRelative( true )
			Player.weapon.chargeSound:setPos( 0,0,1 )
			Player.weapon.chargeSound:start()
		else
			Player.weapon.chargeSound:stop()
		end
		
		Player.weapon.chargeSound:setVolume( ( Player.chargeTimer / Player.weapon.chargeTime ) ^ .5 )
		Player.weapon.chargeSound:setPitch( 1 + ( Player.chargeTimer / Player.weapon.chargeTime * .4 ) )
	end
	
	if Controller:isFiring() and	-- fire
	self.attackTimer == self.weapon.attackTimerMax and
	self.switchWeaponTimer == 0 and
	self.meleeTimer == self.meleeTimerMax and
	Player.ammo[Player.weapon.ammoIndex].count >= Player.weapon.ammoCost and
	Player.chargeTimer == Player.weapon.chargeTime then
	
		Player.ammo[Player.weapon.ammoIndex].count = Player.ammo[Player.weapon.ammoIndex].count - Player.weapon.ammoCost
		Player.chargeTimer = 0
		
		Player.weapon.sound.source:setRelative( true )
		Player.weapon.sound:setPos( 0,0,1 )
		Player.weapon.sound:setPitch( 1 - random() * rsign() * .065 )
		Player.weapon.sound:play()

		local shotCount = self.weapon.shotCount
		local i = 1
		while i <= shotCount do
			i = i + 1
			
			self.attackTimer = 0
		
			local step = .1
			local dist = 20
			
			-- get aim vectors with random offset
			local yawOffset = Player.weapon.aimOffset
			local pitchOffset = Player.weapon.aimOffset
			while math.dist( yawOffset,pitchOffset, 0,0 ) > self.weapon.aimOffset do
				yawOffset = Player.weapon.aimOffset * random() * rsign()
				pitchOffset = Player.weapon.aimOffset * random() * rsign()
			end
			g3d.camera.lookInDirection( nil,nil,nil, self.cam.yaw + yawOffset, self.cam.pitch + pitchOffset )
			
			local vx,vy,vz = g3d.camera.getLookVector()
			g3d.camera.lookInDirection( nil,nil,nil, self.cam.yaw, self.cam.pitch )
			
			
			local startX,startY,startZ = self.x,self.y - self.cam.height,self.z
			
			local endX = vx * dist + startX
			local endY = vy * dist + startY 
			local endZ = vz * dist + startZ
			
			if Player.weapon.projectile then	-- projectile
				local projectileOffset = .3
			
				Projectile:new(
				Player.weapon.projectile, 
				startX + vx * projectileOffset,
				startY + vy * projectileOffset,
				startZ + vz * projectileOffset,
				
				vx,vy,vz
				)
			else	-- hit scan
				-- fire rays
				local wx,wy,wz = Collision.worldRay( startX,startY,startZ, endX,endY,endZ, step )
				local enemy, ex,ey,ez = Collision.entityRay( startX,startY,startZ, endX,endY,endZ, step, Enemy.list )
				local door, dx,dy,dz = Collision.entityRay( startX,startY,startZ, endX,endY,endZ, step, Door.list )
				local glass, gx,gy,gz = Collision.entityRay( startX,startY,startZ, endX,endY,endZ, step, Glass.list )
				local pushWall, px,py,pz = Collision.entityRay( startX,startY,startZ, endX,endY,endZ, step, PushWall.list )
				
				
				local glassDist = math.dist3D( gx,gy,gz, startX,startY,startZ )
				local doorDist = math.dist3D( dx,dy,dz, startX,startY,startZ )
				local worldDist = math.dist3D( wx,wy,wz, startX,startY,startZ )
				local enemyDist = math.dist3D( ex,ey,ez, startX,startY,startZ )
				local pushDist = math.dist3D( px,py,pz, startX,startY,startZ )
				local minDist = min( enemyDist, worldDist, doorDist, glassDist, pushDist )
				
				-- hit
				if minDist == enemyDist and enemy then
					enemy:damage( Player.weapon.damage )
					
					Particle:new( particle_enemyHit,
					ex - vx * particleOffset,
					ey - vy * particleOffset,
					ez - vz * particleOffset
					)
					
					if Player.weapon.penetrating then
						enemy.collisionBox.skipOnShotThisFrame = true
						shotCount = shotCount + 1
					end
				end
				if minDist == worldDist then
					Particle:new( particle_wallHit,
					wx - vx * particleOffset,
					wy - vy * particleOffset,
					wz - vz * particleOffset
					)
				end
				if minDist == doorDist then
					Particle:new( particle_wallHit,
					dx - vx * particleOffset,
					dy - vy * particleOffset,
					dz - vz * particleOffset
					)
				end
				if minDist == glassDist then
					Particle:new( particle_wallHit,
					gx - vx * particleOffset,
					gy - vy * particleOffset,
					gz - vz * particleOffset
					)
				end
				if minDist == pushDist then
					Particle:new( particle_wallHit,
					px - vx * particleOffset,
					py - vy * particleOffset,
					pz - vz * particleOffset
					)
				end
			end
			
		end
	end
	
end

function Player.melee( self )
	local meleeTime = self.meleeGolden and self.meleeGoldenTime or self.meleeTime
	
	self.meleeTimer = min( self.meleeTimer + hdt / meleeTime, self.meleeTimerMax )

	if Controller:isMeleeing() and self.meleeTimer == self.meleeTimerMax then
		self.meleeDir = -self.cam.yaw + rad( 90 )
		self.meleeTimer = 0
		self.meleeSpeed = self.meleeSpeedMax
		self.velocity.y = self.meleeJump
		
		self.meleeSound:play()
		
		for i, e in pairs( Enemy.list ) do
			local vx,vy,vz = g3d.camera.getLookVector()
			vx = vx * self.meleeRange + self.x
			vy = vy * self.meleeRange - self.cam.height
			vz = vz * self.meleeRange + self.z
			
			if e.collisionBox:inBounds( vx,vy,vz ) then
				e:damage( self.meleeGolden and self.meleeGoldenDamage or self.meleeDamage )
			end
		end
	end
	
	self.meleeSpeed = max( 0, self.meleeSpeed - self.meleeDeacceleration * hdt )
	
	self:move( 
	cos( self.meleeDir ) * self.meleeSpeed * hdt,
	0,
	sin( self.meleeDir ) * self.meleeSpeed * hdt )
end

function Player.switchWeapon( self, index )
	local newWeapon = Controller:getSwappedWeapon( index )

	if newWeapon and newWeapon ~= self.weapon and self.switchWeaponTimer == 0 and self.attackTimer == self.weapon.attackTimerMax and contains( self.weapons, newWeapon.name ) then
		self.switchWeaponTimer = 1
		
		self.nextWeapon = newWeapon
	end
	
	if self.switchWeaponTimer < .5 and self.nextWeapon ~= self.weapon then
		self.weapon = self.nextWeapon
		self.attackTimer = self.weapon.attackTimerMax
	end
	
	self.switchWeaponTimer = max( 0, self.switchWeaponTimer - hdt / self.switchWeaponTimerMax )
end

function Player.pause( self )
	if Controller:toggledPause() then
		Game.state = "paused"
	end
end

function Player.update( self )
	self:fall()
	self:look()
	self:parentCamera()
	self:walk()
	
	Pawn.update( self )
	
	self:viewBob()
	self:fire()
	self:melee()
	self:switchWeapon()

	self:pause()
end


function Player.damage( self, damage )
	if self.armor > 0 then
		self.armor = floor( max( 0, self.armor - max( 0, damage * self.armorMult ) ) )
		
		damage = damage * ( 1 - self.armorMult )
	end
	
	self.health = max( 0, self.health - max( 0, damage ) )

	if self.health == 0 then
		self.cam.height = .15
		self:parentCamera()
		Game.state = "gameover"
	end

	local impactFactor = damage / 15
	Juice:shake( .03 * impactFactor )
	Juice:zoom( .95 )
	Juice:tilt( .35 * impactFactor )
	Juice:freeze( 1/ 60 )

	self.hitSound:setPitch( 1 + random() * .1 )
	self.hitSound:play()
end


function file:init()
	Player = Player:new()
end

function file:update()
end

function file:draw()
end

-- [[ RETURN ]] --
return file